@@ -52,12 +52,14 @@ struct imapc_storage_client {
 
 	ARRAY(struct imapc_storage_event_callback) untagged_callbacks;
 
-	char *auth_error;
+	/* IMAPC_COMMAND_STATE_OK if no auth failure (yet), otherwise result to
+	   the LOGIN/AUTHENTICATE command. */
+	enum imapc_command_state auth_failed_state;
+	char *auth_failed_reason;
+
 	/* Authentication reply was received (success or failure) */
 	bool auth_returned:1;
-	/* Authentication failed */
-	unsigned int auth_failed:1;
-	unsigned int destroying:1;
+	bool destroying:1;
 };
 
 struct imapc_storage {
@@ -73,7 +75,7 @@ struct imapc_storage {
 
 	ARRAY(struct imapc_namespace) remote_namespaces;
 
-	unsigned int namespaces_requested:1;
+	bool namespaces_requested:1;
 };
 
 struct imapc_mail_cache {
@@ -136,11 +138,11 @@ struct imapc_mailbox {
 	const char *guid_fetch_field_name;
 	struct imapc_search_context *search_ctx;
 
-	unsigned int selecting:1;
-	unsigned int syncing:1;
-	unsigned int initial_sync_done:1;
-	unsigned int selected:1;
-	unsigned int exists_received:1;
+	bool selecting:1;
+	bool syncing:1;
+	bool initial_sync_done:1;
+	bool selected:1;
+	bool exists_received:1;
 };
 
 struct imapc_simple_context {
@@ -148,12 +150,16 @@ struct imapc_simple_context {
 	int ret;
 };
 
+#define IMAPC_STORAGE(s)	container_of(s, struct imapc_storage, storage)
+#define IMAPC_MAILBOX(s)	container_of(s, struct imapc_mailbox, box)
+
 int imapc_storage_client_create(struct mail_namespace *ns,
 				const struct imapc_settings *imapc_set,
 				const struct mail_storage_settings *mail_set,
 				struct imapc_storage_client **client_r,
 				const char **error_r);
 void imapc_storage_client_unref(struct imapc_storage_client **client);
+bool imapc_storage_client_handle_auth_failure(struct imapc_storage_client *client);
 
 struct mail_save_context *
 imapc_save_alloc(struct mailbox_transaction_context *_t);
